/*
 *
 *    Artistic License
 *
 *    Preamble
 *
 *    The intent of this document is to state the conditions under which a Package may be copied, such that
 *    the Copyright Holder maintains some semblance of artistic control over the development of the
 *    package, while giving the users of the package the right to use and distribute the Package in a
 *    more-or-less customary fashion, plus the right to make reasonable modifications.
 *
 *    Definitions:
 *
 *    "Package" refers to the collection of files distributed by the Copyright Holder, and derivatives
 *    of that collection of files created through textual modification.
 *
 *    "Standard Version" refers to such a Package if it has not been modified, or has been modified
 *    in accordance with the wishes of the Copyright Holder.
 *
 *    "Copyright Holder" is whoever is named in the copyright or copyrights for the package.
 *
 *    "You" is you, if you're thinking about copying or distributing this Package.
 *
 *    "Reasonable copying fee" is whatever you can justify on the basis of media cost, duplication
 *    charges, time of people involved, and so on. (You will not be required to justify it to the
 *    Copyright Holder, but only to the computing community at large as a market that must bear the
 *    fee.)
 *
 *    "Freely Available" means that no fee is charged for the item itself, though there may be fees
 *    involved in handling the item. It also means that recipients of the item may redistribute it under
 *    the same conditions they received it.
 *
 *    1. You may make and give away verbatim copies of the source form of the Standard Version of this
 *    Package without restriction, provided that you duplicate all of the original copyright notices and
 *    associated disclaimers.
 *
 *    2. You may apply bug fixes, portability fixes and other modifications derived from the Public Domain
 *    or from the Copyright Holder. A Package modified in such a way shall still be considered the
 *    Standard Version.
 *
 *    3. You may otherwise modify your copy of this Package in any way, provided that you insert a
 *    prominent notice in each changed file stating how and when you changed that file, and provided that
 *    you do at least ONE of the following:
 *
 *        a) place your modifications in the Public Domain or otherwise make them Freely
 *        Available, such as by posting said modifications to Usenet or an equivalent medium, or
 *        placing the modifications on a major archive site such as ftp.uu.net, or by allowing the
 *        Copyright Holder to include your modifications in the Standard Version of the Package.
 *
 *        b) use the modified Package only within your corporation or organization.
 *
 *        c) rename any non-standard executables so the names do not conflict with standard
 *        executables, which must also be provided, and provide a separate manual page for each
 *        non-standard executable that clearly documents how it differs from the Standard
 *        Version.
 *
 *        d) make other distribution arrangements with the Copyright Holder.
 *
 *    4. You may distribute the programs of this Package in object code or executable form, provided that
 *    you do at least ONE of the following:
 *
 *        a) distribute a Standard Version of the executables and library files, together with
 *        instructions (in the manual page or equivalent) on where to get the Standard Version.
 *
 *        b) accompany the distribution with the machine-readable source of the Package with
 *        your modifications.
 *
 *        c) accompany any non-standard executables with their corresponding Standard Version
 *        executables, giving the non-standard executables non-standard names, and clearly
 *        documenting the differences in manual pages (or equivalent), together with instructions
 *        on where to get the Standard Version.
 *
 *        d) make other distribution arrangements with the Copyright Holder.
 *
 *    5. You may charge a reasonable copying fee for any distribution of this Package. You may charge
 *    any fee you choose for support of this Package. You may not charge a fee for this Package itself.
 *    However, you may distribute this Package in aggregate with other (possibly commercial) programs as
 *    part of a larger (possibly commercial) software distribution provided that you do not advertise this
 *    Package as a product of your own.
 *
 *    6. The scripts and library files supplied as input to or produced as output from the programs of this
 *    Package do not automatically fall under the copyright of this Package, but belong to whomever
 *    generated them, and may be sold commercially, and may be aggregated with this Package.
 *
 *    7. C or perl subroutines supplied by you and linked into this Package shall not be considered part of
 *    this Package.
 *
 *    8. The name of the Copyright Holder may not be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 *    9. THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 *    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 *    MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
package org.chiba.xml.xforms.ui;

import org.apache.commons.jxpath.JXPathContext;
import org.apache.log4j.Category;
import org.chiba.xml.xforms.Model;
import org.chiba.xml.xforms.NamespaceCtx;
import org.chiba.xml.xforms.exception.XFormsException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Implementation of XForms Common Elements
 *
 * @author <a href="mailto:unl@users.sourceforge.net">Ulrich Nicolas Liss&eacute;</a>
 * @version $Id: Common.java,v 1.1 2004/07/30 14:57:56 mvaldes Exp $
 */
public class Common extends BoundElement {
	private static final Category LOGGER = Category.getInstance(Common.class);

	/**
	 * Creates a new Common object.
	 *
	 * @param element the DOM Element
	 * @param model the Model this Common Element belongs to
	 */
	public Common(Element element, Model model) {
		super(element, model);
	}

	// common specific methods

	/**
	 * Checks wether this element is linked to an external resource.
	 * <p>
	 * This element is considered linked if it has a <code>src</code> attribute.
	 *
	 * @return <code>true</code> if this element is linked to an external resource,
	 * otherwise <code>false</code>.
	 */
	public boolean isLinked() {
		return this.element.hasAttributeNS(NamespaceCtx.XFORMS_NS, SRC_ATTRIBUTE);
	}

	/**
	 * Performs element disposal.
	 *
	 * @throws XFormsException if any error occurred during disposal.
	 */
	public void dispose() throws XFormsException {
		if (getLogger().isDebugEnabled()) {
			getLogger().debug(this + " dispose");
		}

		if (isBound()) {
			// NOP
			return;
		}

		if (isLinked()) {
			// NOP
			return;
		}

		disposeChildren();
		disposeSelf();
	}

	// lifecycle methods

	/**
	 * Performs element init.
	 *
	 * @throws XFormsException if any error occurred during init.
	 */
	public void init() throws XFormsException {
		if (getLogger().isDebugEnabled()) {
			getLogger().debug(this + " init");
		}

		if (isBound()) {
			// initialize value from bound instance data
			initializeValue(getInstanceValue());

			return;
		}

		if (isLinked()) {
			// initialize value from link traversal
			String src = this.element.getAttributeNS(NamespaceCtx.XFORMS_NS, SRC_ATTRIBUTE);
			Object result;

			try {
				result = this.container.getConnectorFactory().createURIResolver(src, this.element).resolve();
			} catch (Exception e) {
				// todo: useful error handling
				getLogger().error(this + " init: resolving of URI '" + src + "' failed", e);

				return;
			}

			if (result instanceof Document) {
				initializeValue(((Document)result).getDocumentElement());

				return;
			}

			if (result instanceof Element) {
				initializeValue((Element)result);

				return;
			}

			// todo: useful error handling
			getLogger().error(this + " init: object model '" + result.getClass().getName() +
			                  "' not supported");

			return;
		}

		initializeChildren();
	}

	/**
	 * Performs element update.
	 *
	 * @throws XFormsException if any error occurred during update.
	 */
	public void update() throws XFormsException {
		if (getLogger().isDebugEnabled()) {
			getLogger().debug(this + " update");
		}

		if (isBound()) {
			// update value
			updateValue(getInstanceValue());

			return;
		}

		if (isLinked()) {
			// todo: configurable reload ???
			return;
		}

		updateChildren();
	}

	/**
	 * Returns the logger object.
	 *
	 * @return the logger object.
	 */
	protected Category getLogger() {
		return LOGGER;
	}

	// lifecycle template methods
	protected final void initializeValue(Element value) {
		initializeValue(JXPathContext.newContext(value).getValue("/").toString());
	}

	/**
	 * __UNDOCUMENTED__
     *
     * todo: is this redundant to updateValue ???
	 *
	 * @param value __UNDOCUMENTED__
	 */
	protected final void initializeValue(String value) {
		// remove original children
		Node child;

		while ((child = this.element.getFirstChild()) != null) {
			this.element.removeChild(child);
		}

		// initialize text node
		this.element.appendChild(this.element.getOwnerDocument().createTextNode(value));
	}

	/**
	 * __UNDOCUMENTED__
	 *
	 * @param value __UNDOCUMENTED__
	 */
    private void updateValue(String value) {
		// update text node
		this.element.getFirstChild().setNodeValue(value);
	}
}
//end of class

