package hero.session;
/*
 * 01/02/2004 - 16:36:12
 *
 * UserSessionEJB.java - 
 * Copyright (C) 2004 Ecoo Team
 * valdes@loria.fr
 * 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLightValue;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserPropertyLocalHome;
import hero.interfaces.BnUserPropertyUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.EngineLocal;
import hero.interfaces.EngineLocalHome;
import hero.interfaces.InvalidValueException;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;
import hero.interfaces.UserServiceLocal;
import hero.interfaces.UserServiceLocalHome;
import hero.util.BonitaConfig;
import hero.historic.BonitaTransfer;
import hero.util.EngineException;
import hero.util.HeroException;
import hero.util.EventConstants;
import hero.interfaces.Constants;

import java.util.ArrayList;
import java.util.Map;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.Hashtable;
import java.util.Enumeration;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

/**
 *
 * The User Session Bean, is an stateful session bean that provides the user API to get information on
 * Todo list and started activities and to produce events on activities (start, terminate, cancel).<br>
 * This Session Bean is based on Engine Session Bean: a recursive implementation that manage the previous execution operations
 * and propagates the activity state changes to the activities that are connected to this one.
 *<br><br>
 * The User Session Bean API provides information about user projects and activites (project list,
 * todo list and activity list) and also useful information about project instances or user
 * preferences. With this API users can performs his task/activities by using start, terminate and
 * cancel methods and also terminates workflow processes.
 * <br><br>
 * <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
 * <br>
 * First of all you have to import the User Session files:<br>
 * <br>
 * import hero.interfaces.UserSessionLocalHome;<br>
 * import hero.interfaces.UserSessionLocal;<br>
 * import hero.interfaces.UserSessionHome;<br>
 * import hero.interfaces.UserSession;<br>
 * import hero.interfaces.UserSessionUtil;<br>
 * <br>
 * Now, it is time to create the User Session instance:<br>
 * <br>
 * Like this if you want to use local interfaces:<br><br>
 * UserSessionLocalHome userh = (UserSessionLocalHome)hero.interfaces.UserSessionUtil.getLocalHome();<br>
 * UserSessionLocal usersession = userh.create();<br>
 * <br>
 * or like this if you use remote interfaces:<br>  
 * <br>
 * UserSessionHome userh = (UserSessionHome)hero.interfaces.UserSessionUtil.getHome();<br>
 * UserSession usersession = userh.create();<br>
 * <br>
 * <br>
 * Now you can call all User Sessions methods...  
 * 
 *
 * @ejb:bean name="UserSession"
 *           display-name="UserSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserSession"
 *           local-jndi-name="ejb/hero/UserSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnUser"
 *              ref-name="myhero/BnUser"
 * @ejb.permission role-name="Admin,user,SuperAdmin" 
 *
 * @jonas.bean
 *      ejb-name="UserSession"
 *      jndi-name="ejb/hero/UserSession"
 *
 * 
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 **/

public class UserSessionBean implements SessionBean, EventConstants {
	
	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger trace = Logger.getLogger(UserSessionBean.class);
	
	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------
	
	private SessionContext mContext;
	private BonitaConfig bonitaConfig;
	
	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------
	
	/**
	 * Get user project list. Workflow processes associated to this user (models/cooperative/instances)
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectList() throws HeroException {
		Collection pjts = new ArrayList();
		Collection pva;
		try{
			this.synchronizeUser();
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			trace.info("start by " + mContext.getCallerPrincipal().getName());
			BnUserValue uv = mUser.getBnUserValue();
			pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
		return pva;
	}
	/**
	 * Get project list names for this user (models/cooperative/instances)
	 * @return a String Collection - the projects list names of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectListNames() throws HeroException {
		trace.info("start by " + mContext.getCallerPrincipal().getName());
		//Collection projects = null;
		Collection projectsNames;
		try{
			this.synchronizeUser();
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			//projects = mUser.getBnProjects();
			BnUserValue uv = mUser.getBnUserValue();
			Collection pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
			projectsNames = new ArrayList();
			Iterator i = pva.iterator();
			while (i.hasNext())
				projectsNames.add(((hero.interfaces.BnProjectLightValue) i.next()).getName());
		} catch (javax.naming.NamingException ne) {ne.printStackTrace();
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {fe.printStackTrace();
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
		return projectsNames;
	}
	
	/**
	 * Get user instances list. This method is equivalent to getProjectList but
	 * it only returns the current instances of the user.
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getInstancesList() throws HeroException {
		trace.info("start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		Collection instances = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			if (pro.getType().equals(Constants.Pj.INSTANCE))
				instances.add(pro);
		}
		return instances;
	}
	
	/**
	 * Get user model list. This method is equivalent to getProjectList but
	 * it only returns the current models of the user.
	 * @return a Collection of BnProjectLightValue objects - the model list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getModelList() throws HeroException {
		trace.info("start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		Collection models = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			if (pro.getType().equals(Constants.Pj.MODEL))
				models.add(pro);
		}
		return models;
	}
	
	/**
	 * Get user cooperative projects list. This method is equivalent to getProjectList but
	 * it only returns the current cooperative projects of the user.
	 * @return a Collection of BnProjectLightValue objects - the cooperative projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getCooperativeList() throws HeroException {
		trace.info("start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		Collection coo = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			if (pro.getType().equals(Constants.Pj.COOPERATIVE))
				coo.add(pro);
		}
		return coo;
	}
	
	/**
	 * Get instances list names for this user. This method is equivalent to getProjectListNames but
	 * it only returns the current instances of the user.
	 * @return a String Collection - the projects list names of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getInstancesListNames() throws HeroException {
		trace.info("start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		Collection instances = new ArrayList();
		Collection pva = this.getProjectList();
		Iterator ps = pva.iterator();
		while (ps.hasNext())
		{
			BnProjectLightValue pro = (BnProjectLightValue)ps.next();
			if (pro.getName().matches(".*_instance.*"))
				instances.add(pro.getName());
		}
		return instances;
	}
	
	/**
	 * Delete a Workflow project
	 * @param projectName the name of the project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void removeProject(String projectName) throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameter: projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());		
		BnProjectLocalHome pHome;
		JMSServicesSessionLocal jms;
		this.synchronizeUser();
		try{
			JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
			jms=home.create();
		}catch(Exception e){
			if (this.getProjectConfigTrace(projectName))
				trace.error(e.getMessage());
			throw new EngineException(e.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			BnProjectLocal pl = pHome.findByName(projectName);
			String userName = mContext.getCallerPrincipal().getName();
			if (userName.equals(pl.getCreator()))  // Only project creator must delete the project
			{						
				if (pl.getParent().equals(pl.getName()))
				{	
					Collection ins = new ArrayList();
					if (pl.getModel().getName().equals(pl.getName()))  // is a projectModel
					{
						ins = pl.getInstances();
						Iterator instances = ins.iterator();
						while (instances.hasNext())
						{
							BnProjectLocal pr = (BnProjectLocal)instances.next();
							if (!pr.getName().equals(pl.getName()) && pr.getState() != Constants.Pj.TERMINATED)
								throw new HeroException("Error in delete project: This Workflow model has running instances !!");
						}
					}
					
					Collection nodes = pl.getBnNodes();
					Iterator nds = nodes.iterator();
					while (nds.hasNext())            // Verifies if the project has any subProcesses
					{
						BnNodeLocal nd = (BnNodeLocal)nds.next();
						if (nd.getType()==Constants.Nd.SUB_PROCESS_NODE)
							this.deleteSubProcesses(nd.getName());
					}
					
					if (!ins.isEmpty())
						this.deleteInstances(ins);
					pl.remove();
					jms.sendProjectEvent(DELETEPROJECT, projectName);
				}
				else
					throw new HeroException("Error in delete project: You have to delete subProcess activity of this project !!");
			}
			else
				throw new HeroException("Error in delete project: only the project creator can removed it !!");
		} catch (javax.ejb.FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("User does not exist " + fe.getMessage());
			throw new HeroException("User does not exist");
		} catch (RemoveException re) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Error in delete project " + re.getMessage());
			throw new HeroException("Error in delete project");
		}
	}
	
	/**
	 * Obtain all user activities from specific project (ready and anticipable state)
	 * @param projectName the name of the project
	 * @return a String Collection - the list of todo activities of the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getToDoList(String projectName) throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameter: projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection ready;
		this.synchronizeUser();
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("BnProject " + projectName + " does not exists " + fe.getMessage());
			throw new HeroException("BnProject " + projectName + " does not exists");
		}
		try {
			ready = nodehome.findUserReady(project.getId(), mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (this.getProjectConfigTrace(projectName))
				trace.info("NODE NAME=" + nd.getName());
			result.add(nd.getName());
		}
		return result;
	}
	
	/**
	 * Obtain all user activities from specific project (executing and anticipating state)
	 *
	 * @param projectName the name of the project
	 * @return a String Collectio - the list of active activities of the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getActivityList(String projectName)
	throws HeroException {
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection executing;
		this.synchronizeUser();
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameter: -begin projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("BnProject " + projectName + " does not exists " + fe.getMessage());
			throw new HeroException("BnProject " + projectName + " does not exists");
		}
		try {
			executing =
				nodehome.findUserExecuting(project.getId(), mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(fe.getMessage());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			result.add(nd.getName());
		}
		return result;
	}	
	
	/**
	 * Obtain todo user activities for all instances/cooperative projects (ready and anticipable state)
	 * @return a BnNodeValue Collection - the list of todo activities of the user for all instances
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getToDoListAllInstances() throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection ready;
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ready = nodehome.findUserReadyAllProjects(mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getType().equals(Constants.Pj.INSTANCE) || nd.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}
	
	/**
	 * Obtain executing user activities for all instances/cooperative projects (executing and anticipating state)
	 * @return a BnNodeValue Collection - the list of executing activities ot the user for all instances
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getActivityListAllInstances() throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection executing;
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			executing = nodehome.findUserExecutingAllProjects(mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();				
			if (nd.getBnProject().getType().equals(Constants.Pj.INSTANCE) || nd.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}
	
	/**
	 * Obtain terminated user activities for all instances/cooperative projects.
	 * @return a BnNodeValue Collection - the list of terminated activities ot the user for all instances
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getTerminatedListAllInstances() throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection executing;
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			executing = nodehome.findUserTerminatedAllProjects(mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getType().equals(Constants.Pj.INSTANCE) || nd.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}
	
	
	/**
	 * Obtain todo user activities matching with project property value (ready and anticipable state activities)
	 * @return a BnNodeValue Collection - the list of todo activities matching to project property value
	 * @param key the key of the property
	 * @param value the value of the property
	 * 
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getToDoListByProperty(String key, String value) throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection ready;
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ready = nodehome.findUserReadyAllProjectsByProperty(mContext.getCallerPrincipal().getName(),key,value);
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}
	
	/**
	 * Obtain todo user activities matching with project properties (ready and anticipable state activities)
	 * @return a BnNodeValue Collection - the list of todo activities matching to property value. This method
	 * is oriented to administrative workflow (workflow instances) and supports one, two or three properties (key/value pairs).
	 * Operation constants values: hero.interfaces.Constants.AND, hero.interfaces.Constants.OR 
	 * 
	 * @param operation AND/OR operations are allowed.
	 * @param properties Hashtable containing the key/value elements
	 * 
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getToDoListByProperties(int operation, Hashtable properties) throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection ready = new ArrayList();
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			Enumeration keys = properties.keys();
			Collection vls = properties.values();
			Iterator values = vls.iterator();
			if (properties.size() >3)
			{
				throw new HeroException("This method uses EJBQL language to improve queries. These queries are based " +
				"on static parameters numbers, so if you want use this method with more than three properties contact us !!");
			}
			if (properties.size() == 1)			
				ready = nodehome.findUserReadyAllProjectsByProperty(mContext.getCallerPrincipal().getName(),(String)keys.nextElement(),(String)values.next());
			if (operation == Constants.AND)
			{
				if (properties.size() == 2)			
					ready = nodehome.findUserReadyAllProjectsByTwoPropertiesAND(mContext.getCallerPrincipal().getName(),(String)keys.nextElement(),(String)values.next(),(String)keys.nextElement(),(String)values.next());
				else if (properties.size() == 3)
					ready = nodehome.findUserReadyAllProjectsByThreePropertiesAND(mContext.getCallerPrincipal().getName(),(String)keys.nextElement(),(String)values.next(),(String)keys.nextElement(),(String)values.next(),(String)keys.nextElement(),(String)values.next());
			}
			else if (operation == Constants.OR)
			{
				if (properties.size() == 2)			
					ready = nodehome.findUserReadyAllProjectsByTwoPropertiesOR(mContext.getCallerPrincipal().getName(),(String)keys.nextElement(),(String)values.next(),(String)keys.nextElement(),(String)values.next());
				else if (properties.size() == 3)
					ready = nodehome.findUserReadyAllProjectsByThreePropertiesOR(mContext.getCallerPrincipal().getName(),(String)keys.nextElement(),(String)values.next(),(String)keys.nextElement(),(String)values.next(),(String)keys.nextElement(),(String)values.next());
			}			
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}
	
	/**
	 * Obtain todo user activities matching with activity property value (ready and anticipable state activities)
	 * @return a BnNodeValue Collection - the list of todo activities matching to activity property value
	 * @param key the key of the property
	 * @param value the value of the property
	 * 
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getToDoListByActivityProperty(String key, String value) throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection ready;
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ready = nodehome.findUserReadyAllActivitiesByProperty(mContext.getCallerPrincipal().getName(),key,value);
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}
	
	/**
	 * Obtain executing user activities matching with property value (executing and anticipating state activities)
	 * @return a BnNodeValue Collection - the list of executing activities matching to property value
	 * @param key the key of the property
	 * @param value the value of the property
	 * 
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getActivityListByProperty(String key, String value) throws HeroException {
		trace.info(" start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection executing;
		this.synchronizeUser();
		try {
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			executing = nodehome.findUserExecutingAllProjectsByProperty(mContext.getCallerPrincipal().getName(),key,value);
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (nd.getBnProject().getName().matches(".*_instance.*"))
				result.add(nd.getBnNodeValue());
		}
		return result;
	}	
	/**
	 * Get workflow instances names of this project
	 *
	 * @param projectName the name of the project
	 * @return a String Collection - the names of project instances
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectInstancesNames(String projectName)
	throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameter: projectName = "+ projectName+ " start by "+ mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		this.synchronizeUser();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances =projhome.findUserInstancesByProject(projectName,this.getUser());
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (!pl.getName().equals(projectName))
					result.add(pl.getName());
			}			
			return (result);
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	/**
	 * Get Workflow instances of this project
	 *
	 * @return a BnProjectValue Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getProjectInstances(String projectName) throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameter: projectName = "+projectName + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		this.synchronizeUser();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findUserInstancesByProject(projectName,this.getUser());
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (!pl.getName().equals(projectName))
					result.add(pl.getBnProjectValue());
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}			
	}
	
	/**
	 * Get Workflow instances from a property
	 *
	 * @return a BnProjectValue Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	
	public Collection getInstancesByProperty(String key, String value)
	throws HeroException {
		trace.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		this.synchronizeUser();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (pl.getName().matches(".*_instance.*"))
					result.add(pl.getBnProjectValue());
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * Get Workflow instances from a property
	 *
	 * @return a String Collection - list of project instances
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	
	public Collection getInstancesByPropertyNames(String key, String value)
	throws HeroException {
		trace.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		this.synchronizeUser();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (pl.getName().matches(".*_instance.*"))
					result.add(pl.getName());
				
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * Get Workflow projects from a property
	 *
	 * @return a BnProjectValue Collection - list of projects
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	
	public Collection getProjectsByProperty(String key, String value)
	throws HeroException {
		trace.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		this.synchronizeUser();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (!pl.getName().matches(".*_instance.*"))
					result.add(pl.getBnProjectValue());
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * Get Workflow projects from a property
	 *
	 * @return a String Collection - list of project names
	 * @param projectName the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	
	public Collection getProjectsByPropertyNames(String key, String value)
	throws HeroException {
		trace.info("parameters: key = " + key +" value = " + value + " start by " + mContext.getCallerPrincipal().getName());
		ArrayList result = new ArrayList();
		BnProjectLocalHome projhome;
		this.synchronizeUser();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances = projhome.findProjectsByProperty(key, value);
			for (Iterator i = instances.iterator(); i.hasNext();) {
				BnProjectLocal pl = (BnProjectLocal) i.next();
				if (!pl.getName().matches(".*_instance.*"))
					result.add(pl.getName());
				
			}
			return (result);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * Tries to start an activity (when activity state is ready or anticipable)
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required" 
	 *
	 **/
	
	public void startActivity(String projectName, String nodeName)
	throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameters: projectName="+ projectName+ " nodeName="+ nodeName + " start by " + mContext.getCallerPrincipal().getName());
		EngineLocalHome enghome;
		this.synchronizeUser();
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.startActivity(projectName, nodeName);
			engine.remove();
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (RemoveException r) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot remove Engine bean for "+ projectName+ " " + r.getMessage());
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
		} catch (CreateException c) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot create Engine bean for "+ projectName+ " " + c.getMessage());
			throw new HeroException("Cannot create Engine bean for "+ projectName+ " "+ c.getMessage());
		}	
	}
	
	/**
	 * Tries to terminate an activity (when activity state is executing or anticipating)
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void terminateActivity(String projectName, String nodeName)
	throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameters: projectName="+ projectName+ " nodeName="+ nodeName + " start by " + mContext.getCallerPrincipal().getName());
		EngineLocalHome enghome;
		this.synchronizeUser();
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.terminateActivity(projectName, nodeName);
			engine.remove();
		} catch (RemoveException r) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot remove Engine bean for "+ projectName+ " " + r.getMessage());
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());			
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (CreateException c) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot create Engine bean for " + projectName + " " + c.getMessage());
			throw new HeroException("Cannot create Engine bean for " + projectName);
		}
	}
	
	/**
	 * Tries to cancel an activity (when activity is executing or anticipating)
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void cancelActivity(String projectName, String nodeName)
	throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameters: projectName="+ projectName+ " nodeName="+ nodeName + " start by " + mContext.getCallerPrincipal().getName());
		EngineLocalHome enghome;
		this.synchronizeUser();
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.cancelActivity(projectName, nodeName);
			engine.remove();
		} catch (RemoveException r) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot remove Engine bean for "+ projectName + " " + r.getMessage());
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (CreateException c) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot create Engine bean for " + projectName + " " + c.getMessage());
			throw new EJBException("Cannot create Engine bean for " + projectName);
		}
	}
	/**
	 * Tries to terminate a project (only when all project activities are terminated)
	 * @param projectName the name of the project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 **/
	public void terminate(String projectName) throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameter; projectName=" + projectName + " start by " + mContext.getCallerPrincipal().getName());
		EngineLocalHome enghome;
		this.synchronizeUser();
		try {
			enghome = hero.interfaces.EngineUtil.getLocalHome();
			EngineLocal engine = enghome.create();
			engine.terminate(projectName);
			engine.remove();
			if (this.getProjectConfigHistoric(projectName).equals(Constants.TRANSFER))
				BonitaTransfer.TransferFile(projectName);
			removeProject(projectName);
		} catch (RemoveException r) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot remove Engine bean for "+ projectName+ " " + r.getMessage());
			throw new HeroException("Cannot remove Engine bean for "+ projectName+ " "+ r.getMessage());
		} catch (javax.naming.NamingException ne) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (CreateException c) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot create Engine bean for " + projectName + " " + c.getMessage());
			throw new EJBException("Cannot create Engine bean for " + projectName);
		} catch (EngineException e) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot terminate project " + e.getMessage());
			throw new HeroException("Cannot terminate project " + e.getMessage());
		}
	}
	
	/**
	 * Get Node Value from a specific project
	 * @return a NodeValue object - get node information
	 * @param projectName the name of the project
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnNodeValue getNode(String projectName, String nodeName)
	throws HeroException {
		if (this.getProjectConfigTrace(projectName))
			trace.info("parameters: projectName=" + projectName + " nodeName=" + nodeName + "start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		try {
			ProjectSessionLocalHome projectseshome =ProjectSessionUtil.getLocalHome();
			ProjectSessionLocal project = projectseshome.create();
			project.initProject(projectName);	
			return project.getNodeValue(nodeName);
		} catch (CreateException f) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Project does not exists " + projectName + " " + f.getMessage());
			throw new HeroException("Project does not exists " + projectName);
		} catch (javax.naming.NamingException r) {
			if (this.getProjectConfigTrace(projectName))
				trace.error("Cannot access to project " + r.getMessage());
			throw new HeroException("Cannot access to project " + r.getMessage());
		}
	}
	
	/**
	 * Get the user name
	 * 
	 * @return a String - get user name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUser() {
		trace.info("start by " + mContext.getCallerPrincipal().getName());
		return mContext.getCallerPrincipal().getName();
	}
	
	/**
	 * Get the user light value
	 * 
	 * @return a BnUserLightValue object - get user information
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnUserLightValue getUserLightValue() throws HeroException{
		try{
			trace.info("start by " + mContext.getCallerPrincipal().getName());
			this.synchronizeUser();
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			BnUserLightValue ulv= mUser.getBnUserLightValue();
			UserServiceLocalHome ushome = null;			
			try {
				ushome = hero.interfaces.UserServiceUtil.getLocalHome();
				UserServiceLocal ul = ushome.create();
				Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
				ulv.setEmail((String)infos.get("email"));
				ulv.setPassword((String)infos.get("password"));
				ulv.setJabber((String)infos.get("jabber"));
				return ulv;
			} catch (Exception ce) {
				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
	
	/**
	 * Set the mail of this user into Bonita database.
	 *
	 * @param userName the name of the user
	 * @param userName the mail of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setUserMail(String userName, String mail) throws HeroException {
		trace.info("parameters:: userName = " + userName+" mail = "+mail + " start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		try {
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal user = userhome.findByName(userName);
			user.setEmail(mail);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * Get the mail of this user from Bonita database.
	 *
	 * @param userName the name of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getUserMail(String userName) throws HeroException {
		trace.info("parameter: userName = " + userName + " start by " + mContext.getCallerPrincipal().getName());
		UserServiceLocalHome ushome = null;
		this.synchronizeUser();
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
			return ((String)infos.get("email"));
		} catch (Exception ce) {
			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}				
	}
	
		
	/**
	 * Get the user password
	 *
	 * @return a String - get user password
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserPassword() throws HeroException{
		UserServiceLocalHome ushome = null;
		this.synchronizeUser();
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
			return ((String)infos.get("password"));
		} catch (Exception ce) {
			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}		
	}
	
	/**
	 * Get the user jabber address
	 *
	 * @return a String - get user jabber address
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserJabber() throws HeroException{
		UserServiceLocalHome ushome = null;
		this.synchronizeUser();
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
			return ((String)infos.get("jabber"));
		} catch (Exception ce) {
			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}		
	}
	
	/**
	 * Get User properties
	 *
	 * @return a BnUserPropertyValue Collection - list of user properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getUserProperties() throws HeroException {
		Collection prp = new ArrayList();
		BnUserPropertyLocal pl;
		this.synchronizeUser();
		try{
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			Collection pls = mUser.getBnProperties();
			
			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnUserPropertyLocal) i.next();
				prp.add(pl.getBnUserPropertyValue());
			}			
			return (prp);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
	
	/**
	 * Set a property of a user
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	
	public void setUserProperty(String key, String value)
	throws HeroException {
		BnUserPropertyLocalHome pHome;
		BnUserPropertyLocal propertyLocal = null;
		trace.info("parameters:  key=" + key + " value=" + value + " start by " + mContext.getCallerPrincipal().getName());
		this.synchronizeUser();
		
		try {
			BnUserLocalHome userhome =	hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =	userhome.findByName(mContext.getCallerPrincipal().getName());
			try {
				pHome = BnUserPropertyUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				trace.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
			Collection c = null;
			try {
				c = pHome.findByTheKey(mUser.getId(), key);
			} catch (FinderException fe) {
				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			}
			
			if (!c.isEmpty()) {
				
				propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
				propertyLocal.setTheValue(value);
				
			} else {
				try {
					propertyLocal = pHome.create(key, value);
					propertyLocal.setBnUser(mUser);
					
				} catch (InvalidValueException ie) {
					trace.error(ie.getMessage());
					throw new HeroException(ie.getMessage());
				} catch (javax.ejb.CreateException ce) {
					trace.error(ce.getMessage());
					throw new HeroException(ce.getMessage());
				}
				
			}
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
	
	/**
	 * Delete a property of a user
	 *
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void deleteUserProperty(String key) throws HeroException {
		trace.info("parameter key=" + key + " start by " + mContext.getCallerPrincipal().getName());
		
		BnUserPropertyLocalHome pHome;
		BnUserPropertyLocal propertyLocal = null;
		this.synchronizeUser();
		try {
			BnUserLocalHome userhome =	hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =	userhome.findByName(mContext.getCallerPrincipal().getName());
			try {
				pHome = BnUserPropertyUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				trace.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
			Collection c = null;
			try {
				c = pHome.findByTheKey(mUser.getId(), key);
			} catch (FinderException fe) {
				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			}
			
			if (!c.isEmpty()) {
				try {
					propertyLocal = (BnUserPropertyLocal) (c.toArray())[0];
					propertyLocal.remove();
				} catch (RemoveException rm) {
					trace.error(rm.getMessage());
					throw new HeroException(rm.getMessage());
				}
			} else {
				throw new HeroException("Property not found :" + key);
			}
			
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
	
	private void deleteInstances(Collection instances) throws HeroException{
		Iterator inst = instances.iterator();
		JMSServicesSessionLocal jms;
		this.synchronizeUser();
		try{
			JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
			jms=home.create();
		}catch(Exception e){
			trace.error(e.getMessage());
			throw new EngineException(e.getMessage());
		}
		try{
			while (inst.hasNext())
			{
				BnProjectLocal ins = (BnProjectLocal)inst.next();
				if (ins.getName().matches(".*_instance.*"))
				{
					ins.remove();
					jms.sendProjectEvent(DELETEPROJECT, ins.getName());
				}
			}
		} catch (RemoveException re) {
			trace.error("Error in delete instance " + re.getMessage());
			throw new HeroException("Error in delete project");
		}
	}
	
	private void deleteSubProcesses(String projectName) throws HeroException{
		BnProjectLocalHome pHome;
		JMSServicesSessionLocal jms;
		this.synchronizeUser();
		try{
			JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
			jms=home.create();
		}catch(Exception e){
			if (this.getProjectConfigTrace(projectName))
				trace.error(e.getMessage());
			throw new EngineException(e.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			if (this.getProjectConfigTrace(projectName))
				trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			BnProjectLocal pl = pHome.findByName(projectName);
			String userName = mContext.getCallerPrincipal().getName();
			Collection nodes = pl.getBnNodes();
			Iterator nds = nodes.iterator();
			while (nds.hasNext())            // Verifies if the project has any subProcesses
			{
				BnNodeLocal nd = (BnNodeLocal)nds.next();
				if (nd.getType()==Constants.Nd.SUB_PROCESS_NODE)
					this.deleteSubProcesses(nd.getName());
			}
			pl.remove();
			jms.sendProjectEvent(DELETEPROJECT, projectName);
		} catch (Exception re) {
			trace.error("Error in delete subProcess " + re.getMessage());
			throw new HeroException("Error in delete subProcess");
		}
	}
	
	private boolean getProjectConfigTrace(String projectName) throws HeroException{			
		return (bonitaConfig.getProcessTrace(projectName));
	}         
	
	private String getProjectConfigHistoric(String projectName) throws HeroException{			
		return (bonitaConfig.getPocessHistoric(projectName));
	}         
	
	/**
	 * Creates the User Session Bean. This method is the first one to invoke in order to
	 * use UserSession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {	
	}
	
	private void synchronizeUser() throws HeroException{
		UserServiceLocalHome ushome = null;
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			ul.findUserLocal(mContext.getCallerPrincipal().getName());
		} catch (Exception ce) {ce.printStackTrace();
			trace.error("Error in findUserLocal " + ce.getMessage());
			throw new HeroException("Error in findUserLocal: "+ce.getMessage());
		}
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
			this.bonitaConfig = new BonitaConfig();			
		}catch(Exception e){throw new EJBException(e.getMessage());}
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	public void ejbRemove() {
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	public void ejbActivate() {
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	public void ejbPassivate() {
	}
	
}
